// BWSelectableImageView -- An image view that lets users select regions.

// Copyright (C) 2005 Borkware, LLC
// We use the BSD License.  Check out http://borkware.com/license

#import <Cocoa/Cocoa.h>

extern NSString *BWSelectableImageView_valuePathBinding;
extern NSString *BWSelectableImageView_selectionRectBinding;
extern NSString *BWSelectableImageView_rotationFactorBinding;

typedef enum BWSelectableImageView_DragMode {
    BWSelectableImageView_NotDragging,
    BWSelectableImageView_Normal,
    BWSelectableImageView_Constrain,
    BWSelectableImageView_MoveSelection
} BWSelectableImageView_DragMode;

@interface BWSelectableImageView : NSView
{
    // support two bindings (valuePath and selectionRect)
    // If we support many more, should move these into 
    // dictionaries

    // "valuePath"  (nsstring)
    id valuePathBinding;
    NSString *valuePathKeyPath;

    // "selectionRect" (nsvalue wrapping an nsrect)
    id selectionRectBinding;
    NSString *selectionRectKeyPath;

    // "rotationFactor" (nsnumber, for number of 90-degree rotations CCW)
    id rotationFactorBinding;
    NSString *rotationFactorKeyPath;

    // the image to display.  Gotten when the valuePath binding is set
    NSImage *image;

    // for some reason we get *five* notifications when doing initial
    // binding and when the selection changes in the listview
    NSString *lastImagePath;
    int lastImageRotationFactor;

    // transformation matrix from image space to view space.  In image
    // coordinates
    NSAffineTransform *transform;
    NSAffineTransform *inverse;

    // diagonal points for the selection rectangle.  Also in image
    // coordinates
    NSPoint selectionAnchor;
    NSPoint selectionOpposite;

    NSPoint selectionTrack;

    // used during dragging - shift clicking to keep the selection
    // rectangle square
    BWSelectableImageView_DragMode dragMode;
}

@end // BWSelectableImageView

